@@ -19,23 +19,26 @@
 # Red Hat Author(s): Chris Lumens <clumens@redhat.com>
 #
 
+from gi.repository import Gtk
+
 from collections import namedtuple
-import itertools
 
 from blivet import arch
-from blivet.devices import DASDDevice, FcoeDiskDevice, iScsiDiskDevice, MultipathDevice, MDRaidArrayDevice, ZFCPDiskDevice
+from blivet.devices import DASDDevice, FcoeDiskDevice, iScsiDiskDevice, MultipathDevice, ZFCPDiskDevice
 from blivet.fcoe import has_fcoe
 
 from pyanaconda.flags import flags
-from pyanaconda.i18n import N_, P_
+from pyanaconda.i18n import CN_, CP_
 
-from pyanaconda.ui.lib.disks import getDisks, isLocalDisk, size_str
-from pyanaconda.ui.gui.utils import enlightbox
+from pyanaconda.ui.lib.disks import getDisks
+from pyanaconda.ui.gui.utils import timed_action
 from pyanaconda.ui.gui.spokes import NormalSpoke
 from pyanaconda.ui.gui.spokes.advstorage.fcoe import FCoEDialog
 from pyanaconda.ui.gui.spokes.advstorage.iscsi import ISCSIDialog
+from pyanaconda.ui.gui.spokes.advstorage.zfcp import ZFCPDialog
+from pyanaconda.ui.gui.spokes.advstorage.dasd import DASDDialog
 from pyanaconda.ui.gui.spokes.lib.cart import SelectedDisksDialog
-from pyanaconda.ui.gui.categories.system import SystemCategory
+from pyanaconda.ui.categories.system import SystemCategory
 
 __all__ = ["FilterSpoke"]
 
@@ -43,7 +46,7 @@ DiskStoreRow = namedtuple("DiskStoreRow", ["visible", "selected", "mutable",
                                            "name", "type", "model", "capacity",
                                            "vendor", "interconnect", "serial",
                                            "wwid", "paths", "port", "target",
-                                           "lun", "ccw"])
+                                           "lun", "ccw", "wwpn"])
 
 class FilterPage(object):
     """A FilterPage is the logic behind one of the notebook tabs on the filter
@@ -132,6 +135,18 @@ class FilterPage(object):
         if items:
             combo.set_active(0)
 
+    def _long_identifier(self, disk):
+        # For iSCSI devices, we want the long ip-address:port-iscsi-tgtname-lun-XX
+        # identifier, but blivet doesn't expose that in any useful way and I don't
+        # want to go asking udev.  Instead, we dig around in the deviceLinks and
+        # default to the name if we can't figure anything else out.
+        for link in disk.deviceLinks:
+            if "by-path" in link:
+                lastSlash = link.rindex("/")+1
+                return link[lastSlash:]
+
+        return disk.name
+
 class SearchPage(FilterPage):
     def __init__(self, storage, builder):
         FilterPage.__init__(self, storage, builder)
@@ -183,6 +198,8 @@ class SearchPage(FilterPage):
                 return int(active) == device.node.tpgt
             except ValueError:
                 return True
+        elif active and hasattr(device, "fcp_lun"):
+            return active in device.fcp_lun
         else:
             return True
 
@@ -197,9 +214,7 @@ class SearchPage(FilterPage):
         elif filterBy == 1:
             return self._port_equal(device) and self._target_equal(device) and self._lun_equal(device)
         elif filterBy == 2:
-            return hasattr(device, "wwid") and self._wwidEntry.get_text() in device.wwid
-        elif filterBy == 3:
-            return hasattr(device, "fcp_lun") and self._lunEntry.get_text() in device.fcp_lun
+            return self._wwidEntry.get_text() in getattr(device, "wwid", self._long_identifier(device))
 
     def visible_func(self, model, itr, *args):
         obj = DiskStoreRow(*model[itr])
@@ -229,10 +244,10 @@ class MultipathPage(FilterPage):
             selected = disk.name in selectedNames
 
             store.append([True, selected, not disk.protected,
-                          disk.name, "", disk.model, size_str(disk.size),
+                          disk.name, "", disk.model, str(disk.size),
                           disk.vendor, disk.bus, disk.serial,
                           disk.wwid, "\n".join(paths), "", "",
-                          "", ""])
+                          "", "", ""])
             if not disk.vendor in vendors:
                 vendors.append(disk.vendor)
 
@@ -287,23 +302,12 @@ class OtherPage(FilterPage):
     def ismember(self, device):
         return isinstance(device, iScsiDiskDevice) or isinstance(device, FcoeDiskDevice)
 
-    def _long_identifier(self, disk):
-        # For iSCSI devices, we want the long ip-address:port-iscsi-tgtname-lun-XX
-        # identifier, but blivet doesn't expose that in any useful way and I don't
-        # want to go asking udev.  Instead, we dig around in the deviceLinks and
-        # default to the name if we can't figure anything else out.
-        for link in disk.deviceLinks:
-            if "by-path" in link:
-                lastSlash = link.rindex("/")+1
-                return link[lastSlash:]
-
-        return disk.name
-
     def setup(self, store, selectedNames, disks):
         vendors = []
         interconnects = []
 
         for disk in disks:
+            paths = [d.name for d in disk.parents]
             selected = disk.name in selectedNames
 
             if hasattr(disk, "node"):
@@ -314,10 +318,10 @@ class OtherPage(FilterPage):
                 lun = ""
 
             store.append([True, selected, not disk.protected,
-                          disk.name, "", disk.model, size_str(disk.size),
+                          disk.name, "", disk.model, str(disk.size),
                           disk.vendor, disk.bus, disk.serial,
-                          self._long_identifier(disk), "", port, getattr(disk, "initiator", ""),
-                          lun, ""])
+                          self._long_identifier(disk), "\n".join(paths), port, getattr(disk, "initiator", ""),
+                          lun, "", ""])
 
             if not disk.vendor in vendors:
                 vendors.append(disk.vendor)
@@ -360,52 +364,91 @@ class OtherPage(FilterPage):
         device = self.storage.devicetree.getDeviceByName(obj.name, hidden=True)
         return self.ismember(device) and self._filter_func(device)
 
-class RaidPage(FilterPage):
-    def __init__(self, storage, builder):
-        FilterPage.__init__(self, storage, builder)
-        self.model = self.builder.get_object("raidModel")
-        self.model.set_visible_func(self.visible_func)
-
-    def ismember(self, device):
-        return isinstance(device, MDRaidArrayDevice) and device.isDisk
-
-    def visible_func(self, model, itr, *args):
-        if not flags.dmraid:
-            return False
-
-        obj = DiskStoreRow(*model[itr])
-        device = self.storage.devicetree.getDeviceByName(obj.name, hidden=True)
-        return self.ismember(device)
-
 class ZPage(FilterPage):
     def __init__(self, storage, builder):
         FilterPage.__init__(self, storage, builder)
         self.model = self.builder.get_object("zModel")
         self.model.set_visible_func(self.visible_func)
 
+        self._ccwEntry = self.builder.get_object("zCCWEntry")
+        self._wwpnEntry = self.builder.get_object("zWWPNEntry")
+        self._lunEntry = self.builder.get_object("zLUNEntry")
+        self._combo = self.builder.get_object("zTypeCombo")
+
         self._isS390 = arch.isS390()
 
+    def clear(self):
+        self._lunEntry.set_text("")
+        self._ccwEntry.set_text("")
+        self._wwpnEntry.set_text("")
+
     def ismember(self, device):
         return isinstance(device, ZFCPDiskDevice) or isinstance(device, DASDDevice)
 
     def setup(self, store, selectedNames, disks):
+        """ Set up our Z-page, but only if we're running on s390x. """
         if not self._isS390:
             return
+        else:
+            ccws = []
+            wwpns = []
+            luns = []
+
+            self._combo.set_active(0)
+            self._combo.emit("changed")
+
+            for disk in disks:
+                paths = [d.name for d in disk.parents]
+                selected = disk.name in selectedNames
+
+                if getattr(disk, "type") == "zfcp":
+                    # remember to store all of the zfcp-related junk so we can
+                    # see it in the UI
+                    if not disk.fcp_lun in luns:
+                        luns.append(disk.fcp_lun)
+                    if not disk.wwpn in wwpns:
+                        wwpns.append(disk.wwpn)
+                    if not disk.hba_id in ccws:
+                        ccws.append(disk.hba_id)
+
+                    # now add it to our store
+                    store.append([True, selected, not disk.protected,
+                                  disk.name, "", disk.model, str(disk.size),
+                                  disk.vendor, disk.bus, disk.serial, "", "\n".join(paths),
+                                  "", "", disk.fcp_lun, disk.hba_id, disk.wwpn])
+
+    def _filter_func(self, device):
+        if not self.filterActive:
+            return True
+
+        filterBy = self._combo.get_active()
+
+        if filterBy == 0:
+            return True
+        elif filterBy == 1:
+            return self._ccwEntry.get_text() in device.hba_id
+        elif filterBy == 2:
+            return self._wwpnEntry.get_text() in device.wwpn
+        elif filterBy == 3:
+            return self._lunEntry.get_text() in device.fcp_lun
+
+        return False
 
     def visible_func(self, model, itr, *args):
         obj = DiskStoreRow(*model[itr])
         device = self.storage.devicetree.getDeviceByName(obj.name, hidden=True)
-        return self.ismember(device)
+        return self.ismember(device) and self._filter_func(device)
 
 class FilterSpoke(NormalSpoke):
     builderObjects = ["diskStore", "filterWindow",
-                      "searchModel", "multipathModel", "otherModel", "raidModel", "zModel"]
+                      "searchModel", "multipathModel", "otherModel", "zModel"]
     mainWidgetName = "filterWindow"
     uiFile = "spokes/filter.glade"
+    helpFile = "FilterSpoke.xml"
 
     category = SystemCategory
 
-    title = N_("_INSTALLATION DESTINATION")
+    title = CN_("GUI|Spoke", "_INSTALLATION DESTINATION")
 
     def __init__(self, *args):
         NormalSpoke.__init__(self, *args)
@@ -419,6 +462,11 @@ class FilterSpoke(NormalSpoke):
     def indirect(self):
         return True
 
+    # This spoke has no status since it's not in a hub
+    @property
+    def status(self):
+        return None
+
     def apply(self):
         onlyuse = self.selected_disks[:]
         for disk in [d for d in self.storage.disks if d.name in onlyuse]:
@@ -434,7 +482,6 @@ class FilterSpoke(NormalSpoke):
         self.pages = [SearchPage(self.storage, self.builder),
                       MultipathPage(self.storage, self.builder),
                       OtherPage(self.storage, self.builder),
-                      RaidPage(self.storage, self.builder),
                       ZPage(self.storage, self.builder)]
 
         self._notebook = self.builder.get_object("advancedNotebook")
@@ -442,6 +489,7 @@ class FilterSpoke(NormalSpoke):
         if not arch.isS390():
             self._notebook.remove_page(-1)
             self.builder.get_object("addZFCPButton").destroy()
+            self.builder.get_object("addDASDButton").destroy()
 
         if not has_fcoe():
             self.builder.get_object("addFCOEButton").destroy()
@@ -449,6 +497,27 @@ class FilterSpoke(NormalSpoke):
         self._store = self.builder.get_object("diskStore")
         self._addDisksButton = self.builder.get_object("addDisksButton")
 
+        # Connect focus events in scrolled viewport children to scrolling on the viewport
+        searchScrolledViewport = self.builder.get_object("searchScrolledViewport")
+        searchGrid = self.builder.get_object("searchGrid")
+        searchGrid.set_focus_hadjustment(searchScrolledViewport.get_hadjustment())
+        searchGrid.set_focus_vadjustment(searchScrolledViewport.get_vadjustment())
+
+        multipathViewport = self.builder.get_object("multipathViewport")
+        multipathGrid = self.builder.get_object("multipathGrid")
+        multipathGrid.set_focus_hadjustment(multipathViewport.get_hadjustment())
+        multipathGrid.set_focus_vadjustment(multipathViewport.get_vadjustment())
+
+        otherViewport = self.builder.get_object("otherViewport")
+        otherGrid = self.builder.get_object("otherGrid")
+        otherGrid.set_focus_hadjustment(otherViewport.get_hadjustment())
+        otherGrid.set_focus_vadjustment(otherViewport.get_vadjustment())
+
+        zViewport = self.builder.get_object("zViewport")
+        zGrid = self.builder.get_object("zGrid")
+        zGrid.set_focus_hadjustment(zViewport.get_hadjustment())
+        zGrid.set_focus_vadjustment(zViewport.get_vadjustment())
+
     def _real_ancestors(self, disk):
         # Return a list of all the ancestors of a disk, but remove the disk
         # itself from this list.
@@ -460,15 +529,13 @@ class FilterSpoke(NormalSpoke):
         self.disks = getDisks(self.storage.devicetree)
         self.selected_disks = self.data.ignoredisk.onlyuse[:]
 
-        self.ancestors = itertools.chain(*map(self._real_ancestors, self.disks))
-        self.ancestors = map(lambda d: d.name, self.ancestors)
+        self.ancestors = [d.name for disk in self.disks for d in self._real_ancestors(disk)]
 
         self._store.clear()
 
         allDisks = []
         multipathDisks = []
         otherDisks = []
-        raidDisks = []
         zDisks = []
 
         # Now all all the non-local disks to the store.  Everything has been set up
@@ -476,14 +543,12 @@ class FilterSpoke(NormalSpoke):
         # these lists of disks, then call setup on each individual page.  This is
         # because there could be page-specific setup to do that requires a complete
         # view of all the disks on that page.
-        for disk in itertools.ifilterfalse(isLocalDisk, self.disks):
+        for disk in self.disks:
             if self.pages[1].ismember(disk):
                 multipathDisks.append(disk)
             elif self.pages[2].ismember(disk):
                 otherDisks.append(disk)
             elif self.pages[3].ismember(disk):
-                raidDisks.append(disk)
-            elif self.pages[4].ismember(disk):
                 zDisks.append(disk)
 
             allDisks.append(disk)
@@ -491,8 +556,7 @@ class FilterSpoke(NormalSpoke):
         self.pages[0].setup(self._store, self.selected_disks, allDisks)
         self.pages[1].setup(self._store, self.selected_disks, multipathDisks)
         self.pages[2].setup(self._store, self.selected_disks, otherDisks)
-        self.pages[3].setup(self._store, self.selected_disks, raidDisks)
-        self.pages[4].setup(self._store, self.selected_disks, zDisks)
+        self.pages[3].setup(self._store, self.selected_disks, zDisks)
 
         self._update_summary()
 
@@ -505,11 +569,12 @@ class FilterSpoke(NormalSpoke):
         # potentially show three devices selected (mpatha, sda, sdb for instance).
         count = len([disk for disk in self.selected_disks if disk not in self.ancestors])
 
-        summary = P_("%d _storage device selected",
+        summary = CP_("GUI|Installation Destination|Filter",
+                     "%d _storage device selected",
                      "%d _storage devices selected",
                      count) % count
 
-        label.set_markup("<span foreground='blue'><u>%s</u></span>" % summary)
+        label.set_text(summary)
         label.set_use_underline(True)
 
         summaryButton.set_visible(count > 0)
@@ -527,7 +592,7 @@ class FilterSpoke(NormalSpoke):
         disks = [disk for disk in self.disks if disk.name in self.selected_disks]
         free_space = self.storage.getFreeSpace(disks=disks)
 
-        with enlightbox(self.window, dialog.window):
+        with self.main_window.enlightbox(dialog.window):
             dialog.refresh(disks, free_space, showRemove=False, setBoot=False)
             dialog.run()
 
@@ -536,11 +601,9 @@ class FilterSpoke(NormalSpoke):
         self.pages[n].filterActive = True
         self.pages[n].model.refilter()
 
-    def on_clear_clicked(self, button):
-        n = self._notebook.get_current_page()
-        self.pages[n].filterActive = False
-        self.pages[n].model.refilter()
-        self.pages[n].clear()
+    def on_clear_icon_clicked(self, entry, icon_pos, event):
+        if icon_pos == Gtk.EntryIconPosition.SECONDARY:
+            entry.set_text("")
 
     def on_page_switched(self, notebook, newPage, newPageNum, *args):
         self.pages[newPageNum].model.refilter()
@@ -550,7 +613,10 @@ class FilterSpoke(NormalSpoke):
         if not path:
             return
 
-        itr = self._store.get_iter(path)
+        page_index = self._notebook.get_current_page()
+        filter_model = self.pages[page_index].model
+        model_itr = filter_model.get_iter(path)
+        itr = filter_model.convert_iter_to_child_iter(model_itr)
         self._store[itr][1] = not self._store[itr][1]
 
         if self._store[itr][1] and self._store[itr][3] not in self.selected_disks:
@@ -560,10 +626,15 @@ class FilterSpoke(NormalSpoke):
 
         self._update_summary()
 
+    @timed_action(delay=50, threshold=100)
+    def on_refresh_clicked(self, widget, *args):
+        self.storage.devicetree.populate()
+        self.refresh()
+
     def on_add_iscsi_clicked(self, widget, *args):
         dialog = ISCSIDialog(self.data, self.storage)
 
-        with enlightbox(self.window, dialog.window):
+        with self.main_window.enlightbox(dialog.window):
             dialog.refresh()
             dialog.run()
 
@@ -574,7 +645,7 @@ class FilterSpoke(NormalSpoke):
     def on_add_fcoe_clicked(self, widget, *args):
         dialog = FCoEDialog(self.data, self.storage)
 
-        with enlightbox(self.window, dialog.window):
+        with self.main_window.enlightbox(dialog.window):
             dialog.refresh()
             dialog.run()
 
@@ -583,7 +654,26 @@ class FilterSpoke(NormalSpoke):
         self.refresh()
 
     def on_add_zfcp_clicked(self, widget, *args):
-        pass
+        dialog = ZFCPDialog(self.data, self.storage)
+
+        with self.main_window.enlightbox(dialog.window):
+            dialog.refresh()
+            dialog.run()
+
+        # We now need to refresh so any new disks picked up by adding advanced
+        # storage are displayed in the UI.
+        self.refresh()
+
+    def on_add_dasd_clicked(self, widget, *args):
+        dialog = DASDDialog(self.data, self.storage)
+
+        with self.main_window.enlightbox(dialog.window):
+            dialog.refresh()
+            dialog.run()
+
+        # We now need to refresh so any new disks picked up by adding advanced
+        # storage are displayed in the UI.
+        self.refresh()
 
     ##
     ## SEARCH TAB SIGNAL HANDLERS
@@ -593,10 +683,8 @@ class FilterSpoke(NormalSpoke):
 
         notebook = self.builder.get_object("searchTypeNotebook")
         findButton = self.builder.get_object("searchFindButton")
-        clearButton = self.builder.get_object("searchClearButton")
 
         findButton.set_sensitive(ndx != 0)
-        clearButton.set_sensitive(ndx != 0)
         notebook.set_current_page(ndx)
 
     ##
@@ -607,10 +695,8 @@ class FilterSpoke(NormalSpoke):
 
         notebook = self.builder.get_object("multipathTypeNotebook")
         findButton = self.builder.get_object("multipathFindButton")
-        clearButton = self.builder.get_object("multipathClearButton")
 
         findButton.set_sensitive(ndx != 0)
-        clearButton.set_sensitive(ndx != 0)
         notebook.set_current_page(ndx)
 
     ##
@@ -621,8 +707,18 @@ class FilterSpoke(NormalSpoke):
 
         notebook = self.builder.get_object("otherTypeNotebook")
         findButton = self.builder.get_object("otherFindButton")
-        clearButton = self.builder.get_object("otherClearButton")
 
         findButton.set_sensitive(ndx != 0)
-        clearButton.set_sensitive(ndx != 0)
+        notebook.set_current_page(ndx)
+
+    ##
+    ## Z TAB SIGNAL HANDLERS
+    ##
+    def on_z_type_combo_changed(self, combo):
+        ndx = combo.get_active()
+
+        notebook = self.builder.get_object("zTypeNotebook")
+        findButton = self.builder.get_object("zFindButton")
+
+        findButton.set_sensitive(ndx != 0)
         notebook.set_current_page(ndx)
